import ResizeObserver from 'resize-observer-polyfill';
require('intersection-observer');

(({ behaviors }, loadImage, loadedImage) => {

  function triggerEvent(e) {
    var event = document.createEvent('HTMLEvents');
    event.initEvent('image:size-check', true, false);
    e.dispatchEvent(event);
  }

  function data(e) {
    return JSON.parse(e.getAttribute('data-src'));
  }

  behaviors.imageSizesBehavior = {
    attach(context) {
      const images = Array.from(context.querySelectorAll('img.image-sizes'));
      const imgs =  images.filter(v => !v.classList.contains('load-always'));
      const loadImgs = images.filter(v => v.classList.contains('load-always'));

      images.forEach(v => {
        v.addEventListener('image:size-check', function() {
          imageSizesSetUrl(this);
        })
      })

      let resizeObserver = new ResizeObserver((entries) => {
        entries.forEach((v) => v.target.querySelectorAll('img.image-sizes').forEach(e => triggerEvent(e)))
      });
      images.forEach(v => resizeObserver.observe(v.parentNode));

      let observer = new IntersectionObserver((entries, imageObserver) => {
        entries.forEach((entry) => {
          if (entry.isIntersecting) {
            entry.target.classList.add('is-visible');
            imageObserver.unobserve(entry.target);
            triggerEvent(entry.target);
            resizeObserver.observe(entry.target.parentNode)
          }
        })
      });
      imgs.forEach(v => observer.observe(v));

      loadImgs.forEach(v => triggerEvent(v));
    }
  }
  function setImage(ele, src, width, height) {
    ele.src = src;
    ele.setAttribute('width', width);
    ele.setAttribute('height', height);
    ele.classList.remove('pre-load')
    ele.classList.remove('loading')
    ele.classList.add('fade-in-on-load');
    triggerEvent(ele);
  }

  function shouldLoadingImage(ele) {
    if (ele.classList.contains('load-always') || ele.classList.contains('is-visible')) {
      return true;
    }
    return false;
  }

  function imageSizesSetUrl(ele) {
    if (!shouldLoadingImage(ele)) {
      return;
    }
    var width = parseFloat(getComputedStyle(ele.parentNode, null).width.replace("px"));
    var conf = data(ele);
    var newUrl = false;
    let urlsSet = Object.keys(conf).filter((v) => {
      if (v >= width) {
        return v;
      }
    });
    if (urlsSet.length > 0) {
      newUrl = conf[urlsSet[0]];
    }

    if (newUrl == false) {
      newUrl = ele.getAttribute('data-src-fallback');
    }
    if (newUrl !== ele.src) {
      if (loadedImage.has(newUrl)) {
        const dim = loadedImage.get(newUrl);
        setImage(ele, newUrl, dim.width, dim.height);
        return;
      }
      if (loadImage.has(newUrl)) {
        var eles = loadImage.get(newUrl);
        eles.push(ele);
        ele.classList.add('loading');
        return;
      } else {
        loadImage.set(newUrl, [ele]);
      }
      const img = new Image();
      ele.classList.add('loading');
      img.addEventListener('load', function() {
        const eles = loadImage.get(this.src);
        eles.forEach(e => {
          setImage(e,this.src, img.naturalWidth, img.naturalHeight)
          loadedImage.set(this.src, {
            width: img.naturalWidth,
            height: img.naturalHeight
          });
        })
      });
      img.src = newUrl;
    }
  }
})(Drupal, new Map(), new Map());
