<?php

namespace Drupal\image_sizes\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class ImageSizesPresetEntityForm.
 */
class ImageSizesPresetEntityForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $image_sizes_preset_entity = $this->entity;
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $image_sizes_preset_entity->label(),
      '#description' => $this->t("Label for the Image sizes preset entity."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $image_sizes_preset_entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\image_sizes\Entity\ImageSizesPresetEntity::load',
      ],
      '#disabled' => !$image_sizes_preset_entity->isNew(),
    ];

    $form['preload'] = [
      '#type' => 'select',
      '#default_value' => $image_sizes_preset_entity->getPreloadStyle(),
      '#options' => image_style_options(),
      '#title' => $this->t('Preload style'),
    ];

    $form['styles'] = [
      '#type' => 'checkboxes',
      '#multiple' => TRUE,
      '#default_value' => $image_sizes_preset_entity->getStyles(),
      '#options' => image_style_options(),
      '#title' => $this->t('Styles'),
    ];

    $form['fallback'] = [
      '#type' => 'select',
      '#default_value' => $image_sizes_preset_entity->getFallbackStyle(),
      '#options' => image_style_options(),
      '#title' => $this->t('Fallback style'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function copyFormValuesToEntity(EntityInterface $entity, array $form, FormStateInterface $form_state) {
    parent::copyFormValuesToEntity($entity, $form, $form_state);
    $styles = array_filter($entity->getStyles(), function ($v) {
      return $v !== 0;
    });
    $styles = array_keys($styles);
    $entity->setStyles($styles);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $image_sizes_preset_entity = $this->entity;
    $status = $image_sizes_preset_entity->save();

    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addStatus($this->t('Created the %label Image sizes preset entity.', [
          '%label' => $image_sizes_preset_entity->label(),
        ]));
        break;

      default:
        $this->messenger()->addStatus($this->t('Saved the %label Image sizes preset entity.', [
          '%label' => $image_sizes_preset_entity->label(),
        ]));
    }
    $form_state->setRedirectUrl($image_sizes_preset_entity->toUrl('collection'));
  }

}
